﻿using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WSCF.RevCtrl.Test.Client.ServiceClient;
using WSCF.RevCtrl.Test.IService;

namespace WSCF.RevCtrl.Test.Client
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        WdMainVm viewModel;
        public MainWindow()
        {
            InitializeComponent();

            viewModel = new WdMainVm();
            viewModel.Init(this);

            this.DataContext = viewModel;
        }
    }
    public class WdMainVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public RelayCommand SendCmd { get; private set; }
        public string Addr { get; set; } = "127.0.0.1:5540";
        public string Msg { get; set; }

        public CustomerInfo Selected { get; set; }
        public ObservableCollection<CustomerInfo> AllCustomers { get; } = new ObservableCollection<CustomerInfo>();
        public ObservableCollection<string> Msgs { get; } = new ObservableCollection<string>();
        public Customer Customer => customer;
        public ChatServiceClient Chat => chatService;

        Customer customer;
        ChatServiceClient chatService;
        string connName = "chat";
        MainWindow mainWindow;
        public WdMainVm() {
            SendCmd = new RelayCommand(Send);
        }


        public void Init(MainWindow mainWindow) 
        {
            this.mainWindow = mainWindow;
            //创建本地 顾客 
            customer = new Customer();

            //创建连接 
            //正常情况 FObjServiceClientManager 从配置文件读取
            //最终的目标方便 UnityContainer 使用
            WSCF.FObjServiceClientManager.Instance.DefaultAddrs.Add(new ConnDefaultAddr() { Addr = Addr, ConnName = connName });

            //创建并且连接到 服务器的 Chat 服务
            chatService = new ChatServiceClient("Chat", true, connName);

            //初始化 顾客
            customer.Init(chatService);

            //创建 顾客功能 服务代理， 给服务器反向控制 顾客
            var wsServiceProxy = new WsServiceProxy();
            wsServiceProxy.Init(typeof(ICustomerService), customer);
            chatService.AddProxy("Customer", wsServiceProxy);//挂载服务


            AllCustomers.Add(new CustomerInfo() { Name = $"群聊[{chatService.CustomerCount}]" });
            Selected = AllCustomers[0];

            
            
            chatService.PropertyChanged += ChatService_PropertyChanged;




            //当地址改变后，重新连接服务器
            this.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == nameof(Addr))
                {
                    WSCF.FObjServiceClientManager.Instance.ReConnect(connName, Addr);
                }
            };

            this.customer.Msgs.CollectionChanged += Msgs_CollectionChanged;

        }

        private void Msgs_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action) 
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    mainWindow.Dispatcher.Invoke(() =>
                    {
                        Msgs.Add(e.NewItems[0] as string);
                        if (Msgs.Count > 100)
                            Msgs.RemoveAt(0);
                    });
                    break;
            }
        }



        private void ChatService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(chatService.CustomerInfos))
            {
                updateCustomerInfos(chatService.CustomerInfos);
            }
            else if (e.PropertyName == nameof(chatService.CustomerCount)) {
                AllCustomers[0].Name = $"群聊[{chatService.CustomerCount}]";
            }
        }
        void updateCustomerInfos(CustomerInfo[] customerInfos) 
        {
            //更新 本地有 的信息
            for (int i = 1; i < AllCustomers.Count(); i++) 
            {
                var local_c = AllCustomers[i];
                var remote_cs = from c in customerInfos where c.Guid == local_c.Guid select c;
                if (remote_cs.Count() == 0)
                {
                    //这个不存在，删除
                    mainWindow.Dispatcher.Invoke(() => {
                        AllCustomers.RemoveAt(i);
                    });
                    continue;
                }
                local_c.Name = remote_cs.First().Name;
            }

            //把添加本地没有的信息
            for (int i = 0; i < customerInfos.Count(); i++) 
            {
                var remote_c = customerInfos[i];
                var local_cs = from c in AllCustomers where c.Guid == remote_c.Guid select c;
                if (local_cs.Count() == 0)
                {
                    //这个不存在，添加
                    mainWindow.Dispatcher.Invoke(() => { 
                        AllCustomers.Add(remote_c);
                    });
                    
                }
            }
        }
        private void Send()
        {
            if (string.IsNullOrEmpty(Msg))
                return;

            if (Selected == null || string.IsNullOrEmpty(Selected.Guid))
            {

                chatService.SendMsgToGroupChat(Msg, (asyncContext, retData) =>
                {
                    //成功发送
                    //MessageBox.Show("发送成功");
                }, null);
            }
            else {
                Msgs.Add($"** me ** send to {Selected.Name}:{Msg}");
                chatService.SendMsgToPrivateChat(Selected.Guid, Msg, (asyncContext, retData) =>
                {
                    //成功发送
                    //MessageBox.Show("发送成功");
                }, null);
            }
        }

    }
}
