﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WSCF.Test.IService
{
    /// <summary>
    /// 服务接口 继承INotifyPropertyChanged的，PropertyChanged事件 已经在基类实现，不需要显式 Push() 声明
    /// </summary>
    public interface IFooService:INotifyPropertyChanged
    {

        string Msg{ get; set; }

        int Number { get; }

        DateTime AddFuncTime { get; }

        /// <summary>
        /// 只支持 数组， List, ObservableCollection 都不支持
        /// </summary>
        int[] Abc { get; }


        /// <summary>
        /// ProfileParam 是 INotifyPropertyChanged, 而且它是单例,
        /// 只能通过 PropertyPush 声明, 才能注册 Param.PropertyChanged 事件；
        /// 实现 Param 里面的 属性变化，同步到 客户端与服务器
        /// 这个 Param 里面不能用 method, 它只是 property 的集合体
        /// 当 ProfileParam内还有 property 是 INotifyPropertyChanged 要继续通过 PropertyPush 声明
        /// </summary>
        [PropertyPush]
        ProfileParam Param { get; }

        [PropertyPush]
        Bar[] Bars { get; }
        /// <summary>
        /// 只要是event, 只能是 EventHandler类， 
        /// EventArgs e, 通过 Push() 声明
        /// DataChangedEventArgs 必须是 继承 EventArgs 
        /// </summary>
        [Push(typeof(DataChangedEventArgs))]
        event EventHandler DataChanged;


        /// <summary>
        /// 带返回的方法, 只能通过 回调函数的方法返回。 asyncDelegate
        /// 返回的数据类型 通过 Call() 声明, 对应 asyncDelegate(asyncContext, retData) 中的 retData
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        [Call(typeof(FooServiceAddReponse))]
        void Add(int a, int b, AsyncCBHandler asyncDelegate, object asyncContext);


        /// <summary>
        /// 这个方法 不带返回，会自动 添加 Call() , 不需要手动写
        /// </summary>
        /// <param name="param"></param>
        void Apply(ProfileParam param);


    }
    public class FooServiceAddReponse 
    {
        public int result;
    }
    public class DataChangedEventArgs:EventArgs
    {
        public DateTime Time;
        public string Msg;
    }
    public class ProfileParam : INotifyPropertyChanged
    {

        #region 正常运行参数
        /// <summary>
        /// 产品名称
        /// </summary>
        public string PName { get; set; } = "pname";
        /// <summary>
        /// 目标值
        /// </summary>
        public double Target { get; set; } = 100;

        #endregion

        [PropertyPush]
        public Bar Bar { get; set; } = new Bar();
        /// <summary>
        /// 
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class Bar : INotifyPropertyChanged 
    {
        public int Number { get; set; }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
